"""
Manifests for supported Apps
============================

For now, the following Apps are supported:

default manifest - for Advisor Rules, no special option
-------------------------------------------------------

malware-detection - "--collector malware-detection"
---------------------------------------------------

compliance - "--compliance"
---------------------------


.. note::
    Define the manifest for the App and add to the manifests `dict` at the
    end of this file.
"""

default_manifest = """
---
# version is for the format of this file, not its contents.
version: 0

client:
  context:
    class: insights.core.context.HostContext
    args:
      timeout: 10 # timeout in seconds for commands. Doesn't apply to files.

  # commands and files to ignore
  blacklist:
    files: []
    commands: []
    patterns: []
    keywords: []

  # Can be a list of dictionaries with name/enabled fields or a list of strings
  # where the string is the name and enabled is assumed to be true. Matching is
  # by prefix, and later entries override previous ones. Persistence for a
  # component is disabled by default.
  persist:
    - name: insights.specs.Specs
      enabled: true

  run_strategy:
    name: serial
    args:
      max_workers: null

plugins:
  # disable everything by default
  # defaults to false if not specified.
  default_component_enabled: false

  # packages and modules to load
  packages:
    - insights.specs.default
    - insights.specs.datasources

  # configuration of loaded components. names are prefixes, so any component with
  # a fully qualified name that starts with a key will get the associated
  # configuration applied. Can specify timeout, which will apply to command
  # datasources. Can specify metadata, which must be a dictionary and will be
  # merged with the components' default metadata.
  configs:
    - name: insights.specs.Specs
      enabled: true
    - name: insights.specs.default.DefaultSpecs
      enabled: true
    - name: insights.specs.datasources
      enabled: true
    # needed for specs that aren't given names before used in DefaultSpecs
    - name: insights.core.spec_factory
      enabled: true

    # needed by multiple datasource specs/components
    - name: insights.parsers.hostname.Hostname
      enabled: true
    - name: insights.parsers.hostname.HostnameDefault
      enabled: true
    - name: insights.combiners.hostname.Hostname
      enabled: true
    - name: insights.parsers.uname.Uname
      enabled: true
    - name: insights.parsers.redhat_release.RedhatRelease
      enabled: true
    - name: insights.combiners.redhat_release.RedHatRelease
      enabled: true
    - name: insights.parsers.ps.PsAuxcww
      enabled: true
    - name: insights.parsers.ps.PsAuxww
      enabled: true
    - name: insights.combiners.ps.Ps
      enabled: true
    - name: insights.parsers.dmidecode.DMIDecode
      enabled: true
    - name: insights.parsers.installed_rpms.InstalledRpms
      enabled: true
    - name: insights.parsers.mount.ProcMounts
      enabled: true

    # needed for identifying RHEL major version
    - name: insights.components.rhel_version.IsRhel6
      enabled: true
    - name: insights.components.rhel_version.IsRhel7
      enabled: true
    - name: insights.components.rhel_version.IsRhel8
      enabled: true
    - name: insights.components.rhel_version.IsRhel9
      enabled: true

    # needed for cloud specs
    - name: insights.parsers.yum.YumRepoList
      enabled: true
    - name: insights.parsers.rhsm_conf.RHSMConf
      enabled: true
    - name: insights.combiners.cloud_provider.CloudProvider
      enabled: true
    - name: insights.components.cloud_provider.IsAWS
      enabled: true
    - name: insights.components.cloud_provider.IsAzure
      enabled: true
    - name: insights.components.cloud_provider.IsGCP
      enabled: true

    # needed for ceph specs
    - name: insights.components.ceph.IsCephMonitor
      enabled: true

    # needed for pcp specs
    - name: insights.parsers.systemd.unitfiles.UnitFiles
      enabled: true
    - name: insights.parsers.ros_config.RosConfig
      enabled: true

    # needed for 'teamdctl_config/state_dump' spec and nmcli_conn_show_uuids spec
    - name: insights.parsers.nmcli.NmcliConnShow
      enabled: true

    # needed for mssql_tls_cert_enddate
    - name: insights.parsers.mssql_conf.MsSQLConf
      enabled: true

    # needed for rsyslog_tls_cert_file
    - name: insights.parsers.rsyslog_conf.RsyslogConf
      enabled: true
    - name: insights.combiners.rsyslog_confs.RsyslogAllConf
      enabled: true

    # needed for sap specs
    - name: insights.parsers.saphostctrl.SAPHostCtrlInstances
      enabled: true
    - name: insights.combiners.sap.Sap
      enabled: true

    # needed for specs: fw_security, smartctl_health
    - name: insights.parsers.virt_what.VirtWhat
      enabled: true
    - name: insights.combiners.virt_what.VirtWhat
      enabled: true
    - name: insights.components.virtualization.IsBareMetal
      enabled: true

    # needed for 'modinfo_filtered_modules' spec
    - name: insights.parsers.lsmod.LsMod
      enabled: true

    # needed for satellite server specs
    - name: insights.combiners.satellite_version.SatelliteVersion
      enabled: true
    - name: insights.combiners.satellite_version.CapsuleVersion
      enabled: true
    - name: insights.components.satellite.IsSatellite
      enabled: true
    - name: insights.components.satellite.IsSatellite614AndLater
      enabled: true
    - name: insights.components.satellite.IsSatelliteLessThan614
      enabled: true
    - name: insights.components.satellite.IsSatellite611
      enabled: true
    - name: insights.components.satellite.IsCapsule
      enabled: true

    # needed for container specs
    - name: insights.parsers.podman_list.PodmanListContainers
      enabled: true
    - name: insights.parsers.docker_list.DockerListContainers
      enabled: true

    # needed for specs: luks_data_sources, smartctl_health, 'fstab_mounted.devices' to 'ls_lH_files'
    #  'fstab_mounted.devices' to 'ls_lH_files''
    - name: insights.parsers.blkid.BlockIDInfo
      enabled: true
    - name: insights.components.cryptsetup.HasCryptsetupWithTokens
      enabled: true
    - name: insights.components.cryptsetup.HasCryptsetupWithoutTokens
      enabled: true

    # needed for 'iris' specs
    - name: insights.parsers.iris.IrisList
      enabled: true
    - name: insights.parsers.iris.IrisCpf
      enabled: true

    # needed for ausearch_insights
    - name: insights.components.rhel_version.IsGtOrRhel86
      enabled: true

    # needed for spec: subscription_manager_syspurpose
    - name: insights.components.rhel_version.IsGtOrRhel84
      enabled: true

    # needed for sealert spec
    - name: insights.parsers.selinux_config.SelinuxConfig
      enabled: true
    - name: insights.components.selinux.SELinuxEnabled
      enabled: true

    # needed for 'fstab_mounted.dirs' to 'ls_lan', 'fstab_mounted.devices' to 'ls_lH_files'
    - name: insights.parsers.fstab.FSTab
      enabled: true

    # needed for 'pvs.devices' to 'ls_lH_files' spec
    - name: insights.parsers.lvm.Pvs
      enabled: true
""".strip()


malware_detection_manifest = """
# Manifest file for malware detection app data collection
---
# version is for the format of this file, not its contents.
version: 0

client:
  context:
    class: insights.core.context.HostContext
    args:
      timeout: 10 # timeout in seconds for commands. Doesn't apply to files.

  blacklist:
    files: []
    commands: []
    patterns: []
    keywords: []

  persist:
    # determines what will appear in the archive
    - name: insights.specs.default.DefaultSpecs.malware_detection
      enabled: true

  run_strategy:
    name: serial
    args:
      max_workers: null

plugins:
  # disable everything by default
  # defaults to false if not specified.
  default_component_enabled: false
  packages:
    # determines which packages are loaded. These will be namespaced to the relevant collector
    - insights.specs.datasources.malware_detection
    - insights.specs.default
  configs:
    # determines which specs get loaded
    - name: insights.specs.default.DefaultSpecs.malware_detection
      enabled: true
    # Enable specs for collecting the system's canonical facts
    - name: insights.specs.default.DefaultSpecs.mac_addresses
      enabled: true
    - name: insights.specs.Specs.mac_addresses
      enabled: true
    - name: insights.specs.default.DefaultSpecs.etc_machine_id
      enabled: true
    - name: insights.specs.Specs.etc_machine_id
      enabled: true
    - name: insights.specs.default.DefaultSpecs.hostname
      enabled: true
    - name: insights.specs.Specs.hostname
      enabled: true
    - name: insights.specs.default.DefaultSpecs.dmidecode
      enabled: true
    - name: insights.specs.Specs.dmidecode
      enabled: true
    - name: insights.specs.default.DefaultSpecs.machine_id
      enabled: true
    - name: insights.specs.Specs.machine_id
      enabled: true
    - name: insights.specs.default.DefaultSpecs.ip_addresses
      enabled: true
    - name: insights.specs.Specs.ip_addresses
      enabled: true
    - name: insights.specs.default.DefaultSpecs.subscription_manager_id
      enabled: true
    - name: insights.specs.Specs.subscription_manager_id
      enabled: true
""".lstrip()

compliance_manifest = """
# Manifest file for compliance data collection
---
# version is for the format of this file, not its contents.
version: 0

client:
  context:
    class: insights.core.context.HostContext
    args:
      timeout: 10 # timeout in seconds for commands. Doesn't apply to files.

  blacklist:
    files: []
    commands: []
    patterns: []
    keywords: []

  persist:
    # determines what will appear in the archive
    - name: insights.specs.Specs.compliance
      enabled: true
    - name: insights.specs.Specs.compliance_policies
      enabled: true
    - name: insights.specs.Specs.compliance_assign
      enabled: true
    - name: insights.specs.Specs.compliance_unassign
      enabled: true

  run_strategy:
    name: serial
    args:
      max_workers: null

plugins:
  # disable everything by default
  # defaults to false if not specified.
  default_component_enabled: false
  packages:
    # determines which packages are loaded. These will be namespaced to the relevant collector
    - insights.specs.datasources.compliance
    - insights.specs.default
  configs:
    # determines which specs get loaded
    - name: insights.specs.datasources.compliance.compliance_ds
      enabled: true
    - name: insights.specs.Specs.compliance
      enabled: true
    - name: insights.specs.default.DefaultSpecs.compliance
      enabled: true
    - name: insights.specs.default.DefaultSpecs.compliance_policies
      enabled: true
    - name: insights.specs.default.DefaultSpecs.compliance_assign
      enabled: true
    - name: insights.specs.default.DefaultSpecs.compliance_unassign
      enabled: true

    # Enable specs for collecting the system's canonical facts
    - name: insights.specs.default.DefaultSpecs.mac_addresses
      enabled: true
    - name: insights.specs.Specs.mac_addresses
      enabled: true
    - name: insights.specs.default.DefaultSpecs.etc_machine_id
      enabled: true
    - name: insights.specs.Specs.etc_machine_id
      enabled: true
    - name: insights.specs.default.DefaultSpecs.hostname
      enabled: true
    - name: insights.specs.Specs.hostname
      enabled: true
    - name: insights.specs.default.DefaultSpecs.dmidecode
      enabled: true
    - name: insights.specs.Specs.dmidecode
      enabled: true
    - name: insights.specs.default.DefaultSpecs.machine_id
      enabled: true
    - name: insights.specs.Specs.machine_id
      enabled: true
    - name: insights.specs.default.DefaultSpecs.ip_addresses
      enabled: true
    - name: insights.specs.Specs.ip_addresses
      enabled: true
    - name: insights.specs.default.DefaultSpecs.subscription_manager_id
      enabled: true
    - name: insights.specs.Specs.subscription_manager_id
      enabled: true
    - name: insights.specs.Specs.redhat_release
      enabled: true
    - name: insights.specs.default.DefaultSpecs.redhat_release
      enabled: true
    - name: insights.specs.Specs.os_release
      enabled: true
    - name: insights.specs.default.DefaultSpecs.os_release
      enabled: true
    - name: insights.parsers.redhat_release.RedhatRelease
      enabled: true
    - name: insights.parsers.os_release.OsRelease
      enabled: true
    - name: insights.specs.Specs.installed_rpms
      enabled: true
    - name: insights.specs.default.DefaultSpecs.installed_rpms
      enabled: true
    - name: insights.parsers.installed_rpms.InstalledRpms
      enabled: true
""".lstrip()

manifests = {
    'default': default_manifest,
    'compliance': compliance_manifest,
    'malware-detection': malware_detection_manifest,
}
"""
Pre-defined manifests for different applications.

    :meta hide-value:
"""
content_types = {
    'default': 'application/vnd.redhat.advisor.collection+tgz',
    'compliance': 'application/vnd.redhat.compliance.something+tgz',
    'malware-detection': 'application/vnd.redhat.malware-detection.results+tgz',
}
"""
Specific content types for uploading data for different applications.

    :meta hide-value:
"""



from mock.mock import patch

from insights.client.config import InsightsConfig
from insights.client.core_collector import CoreCollector


@patch('insights.client.core_collector.InsightsArchive')
def test_archive_returned(_):
    c = InsightsConfig()
    d = CoreCollector(c)
    ret = d.done()
    d.archive.create_tar_file.assert_called_once()
    assert ret == d.archive.create_tar_file.return_value


@patch('insights.client.core_collector.InsightsArchive')
def test_dir_returned(_):
    c = InsightsConfig(output_dir='test')
    d = CoreCollector(c)
    ret = d.done()
    d.archive.create_tar_file.assert_not_called()
    assert ret == d.archive.archive_dir



from datetime import datetime

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.rhsm_log)
class RhsmLog(LogFileOutput):
    """
    Class for parsing the log file: ``/var/log/rhsm/rhsm.log``.

    Sample input::
        2016-07-31 04:06:41,215 [DEBUG] rhsmcertd-worker:24440 @identity.py:131 - Loading consumer info from identity certificates.
        2016-07-31 04:06:41,221 [DEBUG] rhsmcertd-worker:24440 @connection.py:475 - Loaded CA certificates from /etc/rhsm/ca/: redhat-uep.pem
        2016-07-31 04:06:41,221 [DEBUG] rhsmcertd-worker:24440 @connection.py:523 - Making request: GET /subscription/consumers/a808d48e-36bf-4071-a00a-0efacc511b2b/certificates/serials
        2016-07-31 04:07:21,245 [ERROR] rhsmcertd-worker:24440 @entcertlib.py:121 - [Errno -2] Name or service not known

    Examples:
        >>> log = rhsm_log.get('Name or service not known')[0]
        >>> log.get('raw_message')
        '2016-07-31 04:07:21,245 [ERROR] rhsmcertd-worker:24440 @entcertlib.py:121 - [Errno -2] Name or service not known'
        >>> log.get('message')
        '[ERROR] rhsmcertd-worker:24440 @entcertlib.py:121 - [Errno -2] Name or service not known'
        >>> log.get("timestamp")
        datetime.datetime(2016, 7, 31, 4, 7, 21, 245000)
    """
    time_format = '%Y-%m-%d %H:%M:%S,%f'

    def _parse_line(self, line):
        """
        Parse log line to valid components
        """
        msg_info = {'raw_message': line}
        line_split = line.split(None, 2)
        try:
            msg_info['timestamp'] = datetime.strptime(' '.join(line_split[:2]), self.time_format)
            msg_info['message'] = line_split[2]
        except (ValueError, IndexError):
            pass
        return msg_info



"""
IsOpenStackCompute
==================

The ``IsOpenStackCompute`` component uses ``PsAuxcww`` parser to determine
OpenStack Compute node. It checks if 'nova-compute' process exist, if not raises
``SkipComponent`` so that the dependent component will not fire. Can be added as
a dependency of a parser so that the parser only fires if the
``IsIsOpenStackCompute`` dependency is met.
"""
from insights.core.exceptions import SkipComponent
from insights.core.plugins import component
from insights.parsers.ps import PsAuxcww


@component(PsAuxcww)
class IsOpenStackCompute(object):
    """The ``IsOpenStackCompute`` component uses ``PsAuxcww`` parser to determine
    OpenStack Compute node. It checks if ``nova-compute`` process exist, if not
    raises ``SkipComponent``.

    Raises:
        SkipComponent: When ``nova-compute`` process does not exist.
    """
    def __init__(self, ps):
        if 'nova-compute' not in ps.running:
            raise SkipComponent('Not OpenStack Compute node')



# -*- coding: UTF-8 -*-

from insights.client.phase.v1 import collect_and_output
from mock.mock import patch
from pytest import raises


def patch_insights_config(old_function):
    patcher = patch("insights.client.phase.v1.InsightsConfig",
                    **{"return_value.load_all.return_value.status": False,
                       "return_value.load_all.return_value.unregister": False,
                       "return_value.load_all.return_value.offline": False,
                       "return_value.load_all.return_value.enable_schedule": False,
                       "return_value.load_all.return_value.disable_schedule": False,
                       "return_value.load_all.return_value.analyze_container": False,
                       "return_value.load_all.return_value.display_name": False,
                       "return_value.load_all.return_value.to_stdout": False,
                       "return_value.load_all.return_value.no_upload": False,
                       "return_value.load_all.return_value.to_json": False,
                       "return_value.load_all.return_value.keep_archive": False,
                       "return_value.load_all.return_value.register": False,
                       "return_value.load_all.return_value.diagnosis": None,
                       "return_value.load_all.return_value.payload": None,
                       "return_value.load_all.return_value.manifest": None,
                       "return_value.load_all.return_value.content_type": None,
                       "return_value.load_all.return_value.compliance": False,
                       "return_value.load_all.return_value.compliance_policies": False,
                       "return_value.load_all.return_value.compliance_assign": None,
                       "return_value.load_all.return_value.compliance_unassign": None,
                       "return_value.load_all.return_value.module": False,
                       "return_value.load_all.return_value.output_dir": None,
                       "return_value.load_all.return_value.output_file": None})
    return patcher(old_function)


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_collect_and_output_payload_on(insights_config, insights_client):
    """
    Collection is not done when a payload is uploaded
    Payload is sent to upload function
    Archive is not deleted
    """
    insights_config.return_value.load_all.return_value.payload = 'testp'
    insights_config.return_value.load_all.return_value.content_type = 'testct'
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.return_value.collect.assert_not_called()
    insights_client.return_value.upload.assert_called_with(payload='testp', content_type='testct')
    insights_client.return_value.delete_cached_branch_info.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_collect_and_output_payload_off(insights_config, insights_client):
    """
    Collection is done and uploaded in normal operation (no payload)
    Archive deleted after upload
    """
    insights_config.return_value.load_all.return_value.payload = False
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.return_value.collect.assert_called_once()
    insights_client.return_value.upload.assert_called_with(
        payload=insights_client.return_value.collect.return_value,
        content_type='application/vnd.redhat.advisor.collection+tgz')
    insights_client.return_value.delete_cached_branch_info.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_collect_and_output_payload_on_default(insights_config, insights_client):
    insights_config.return_value.load_all.return_value.content_type = 'bz2'
    insights_config.return_value.load_all.return_value.payload = 'testct.bz2'
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.return_value.collect.assert_not_called()
    insights_client.return_value.upload.assert_called_with(payload='testct.bz2', content_type='application/vnd.redhat.advisor.collection+bz2')
    insights_client.return_value.delete_cached_branch_info.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_collect_and_output_different_payload_type(insights_config, insights_client):
    insights_config.return_value.load_all.return_value.content_type = 'bz2'
    insights_config.return_value.load_all.return_value.payload = 'testct.tgz'
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.return_value.collect.assert_not_called()
    insights_client.return_value.upload.assert_not_called()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_collect_and_output_none_payload_type(insights_config, insights_client):
    insights_config.return_value.load_all.return_value.payload = 'testct'
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.return_value.collect.assert_not_called()
    insights_client.return_value.upload.assert_called_with(payload='testct', content_type=None)
    insights_client.return_value.delete_cached_branch_info.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
# @patch("insights.client.phase.v1.InsightsClient")
def test_exit_ok(insights_config, insights_client):
    """
    Support collection replaces the normal client run.
    """
    with raises(SystemExit) as exc_info:
        collect_and_output()
    assert exc_info.value.code == 0


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_no_upload(insights_config, insights_client):
    '''
    Copy collected data to a specified file and do not upload.
    '''
    insights_config.return_value.load_all.return_value.no_upload = True
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.upload.assert_not_called()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_output_file(insights_config, insights_client):
    '''
    Copy collected data to a specified file and do not upload.
    '''
    insights_config.return_value.load_all.return_value.no_upload = True
    insights_config.return_value.load_all.return_value.output_file = '/var/tmp/test.tar.gz'
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.return_value.copy_to_output_file.assert_called_with(insights_client.return_value.collect.return_value)


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_output_dir(insights_config, insights_client):
    '''
    Copy collected data to a specified directory and do not upload.
    '''
    insights_config.return_value.load_all.return_value.no_upload = True
    insights_config.return_value.load_all.return_value.output_dir = '/var/tmp/test'
    try:
        collect_and_output()
    except SystemExit:
        pass
    insights_client.return_value.copy_to_output_dir.assert_called_with(insights_client.return_value.collect.return_value)



"""
NTP sources - remote clock info from ``ntpq`` and ``chronyc``
=============================================================

The parsers here provide information about the time sources used by
``ntpd`` and ``chronyd``.  These are gathered from the output of the
``ntpq -pn`` and ``chronyc sources`` commands respectively.

There is also a parser for parsing the output of ``ntpq -c 'rv 0 leap'``
command to give leap second status.

Parsers in this module are:

ChronycSources - command ``/usr/bin/chronyc sources``
-----------------------------------------------------

NtpqPn - command ``/usr/sbin/ntpq -pn``
---------------------------------------

NtpqLeap - command ``/usr/sbin/ntpq -c 'rv 0 leap'``
----------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.chronyc_sources)
class ChronycSources(CommandParser, list):
    """
    Chronyc Sources parser

    Parses the list of NTP time sources in use by ``chronyd``.  So far only
    the source IP address and the mode and the state flags are retrieved.

    Sample input::

        210 Number of sources = 6
        MS Name/IP address         Stratum Poll Reach LastRx Last sample
        ===============================================================================
        ^- 10.20.30.40                   2   9   377    95  -1345us[-1345us] +/-   87ms
        ^- 10.56.72.8                    2  10   377   949  -3449us[-3483us] +/-  120ms
        ^* 10.64.108.95                  2  10   377   371    -91us[ -128us] +/-   30ms
        ^- 10.8.205.17                   2   8   377    27  +7161us[+7161us] +/-   52ms

    Examples:

        >>> type(chrony_sources)
        <class 'insights.parsers.ntp_sources.ChronycSources'>
        >>> len(chrony_sources)
        4
        >>> chrony_sources[0]['source']
        '10.20.30.40'
        >>> chrony_sources[0]['mode']
        '^'
        >>> chrony_sources[0]['state']
        '-'
    """

    def parse_content(self, content):
        """
        Get source, mode and state for chrony
        """
        data = []
        if len(content) > 3:
            for row in content[3:]:
                if row.strip():
                    values = row.split(" ", 2)
                    data.append(
                        {
                            "source": values[1],
                            "mode": values[0][0],
                            "state": values[0][1]
                        }
                    )

        if not data:
            raise SkipComponent()

        self.extend(data)

    @property
    def data(self):
        """
        Set data as property to keep compatibility
        """
        return self


@parser(Specs.ntpq_leap)
class NtpqLeap(CommandParser, dict):
    """
    Converts the output of ``ntpq -c 'rv 0 leap'`` into a dictionary in the
    ``data`` property, and sets the ``leap`` property to the value of the
    'leap' key if found.

    Sample input::

        leap=00

    Examples:

        >>> type(ntpq)
        <class 'insights.parsers.ntp_sources.NtpqLeap'>
        >>> ntpq.leap
        '00'
    """

    def parse_content(self, content):
        if content and "Connection refused" in content[0]:
            raise SkipComponent("NTP service is down and connection refused")

        leap = None
        for line in content:
            if 'leap=' in line:
                leap = line.split('leap=')[1].rstrip()

        if leap is None:
            raise SkipComponent()

        self.update(leap=leap)

    @property
    def data(self):
        """
        Set data as property to keep compatibility
        """
        return self

    @property
    def leap(self):
        """
        Return the value of the 'leap'
        """
        return self.get('leap')


@parser(Specs.ntpq_pn)
class NtpqPn(CommandParser, list):
    """
    Get source and flag for each NTP time source from the output of
    ``/usr/sbin/ntpq -pn``.

    Currently, this only captures the source IP address and the 'flag'
    character in the first column at this stage.  Therefore it will need
    to be extended should you wish to determine the stratum, polling rate
    or other properties of the source.

    Sample input::

             remote           refid      st t when poll reach   delay   offset  jitter
        ==============================================================================
        +10.20.30.40     192.231.203.132  3 u  638 1024  377    0.242    2.461   1.886
        *2001:388:608c:8 .GPS.            1 u  371 1024  377   29.323    1.939   1.312
        -2001:44b8:1::1  216.218.254.202  2 u  396 1024  377   37.869   -3.340   6.458
        +150.203.1.10    202.6.131.118    2 u  509 1024  377   20.135    0.800   3.260

    Examples:

        >>> type(ntp_sources)
        <class 'insights.parsers.ntp_sources.NtpqPn'>
        >>> len(ntp_sources)
        4
        >>> ntp_sources[0]['source']
        '10.20.30.40'
    """

    def parse_content(self, content):
        if content and "Connection refused" in content[0]:
            raise SkipComponent("NTP service is down and connection refused")

        data = []
        if len(content) > 2:
            for row in content[2:]:
                if row.strip():
                    values = row.split(" ", 2)
                    if row.startswith(" "):
                        data.append({"source": values[1], "flag": " "})
                    else:
                        data.append({"source": values[0][1:], "flag": values[0][0]})

        if not data:
            raise SkipComponent()

        self.extend(data)

    @property
    def data(self):
        """
        Set data as property to keep compatibility
        """
        return self



"""
Exceptions
==========
"""

from insights.parsr.iniparser import NoOptionError, NoSectionError  # noqa: F401


class BlacklistedSpec(Exception):
    """
    Exception to be thrown when a blacklisted spec is found.
    """

    pass


class CalledProcessError(Exception):
    """
    Raised if call fails.

    Parameters:
        returncode (int): The return code of the process executing the command.
        cmd (str): The command that was executed.
        output (str): Any output the command produced.
    """

    def __init__(self, returncode, cmd, output=None):
        self.returncode = returncode
        self.cmd = cmd
        self.output = output
        super(CalledProcessError, self).__init__(returncode, cmd, output)

    def __unicode__(self):
        name = self.__class__.__name__
        rc = self.returncode
        cmd = self.cmd
        output = self.output
        return '<{}({}, {!r}, {!r})>'.format(name, rc, cmd, output)


class InvalidArchive(Exception):
    """
    Raised when archive cannot be identified or missing expected structure.
    """

    def __init__(self, msg):
        super(InvalidArchive, self).__init__(msg)
        self.msg = msg


class InvalidContentType(InvalidArchive):
    """
    Raised when invalid content_type is specified.
    """

    def __init__(self, content_type):
        msg = 'Invalid content type: "%s"' % content_type
        super(InvalidContentType, self).__init__(msg)
        self.msg = msg
        self.content_type = content_type


class MissingRequirements(Exception):
    """
    Raised during evaluation if a component's dependencies aren't met.
    """

    def __init__(self, requirements):
        self.requirements = requirements
        super(MissingRequirements, self).__init__(requirements)


class NoFilterException(Exception):
    """
    Raised whenever no filters added to a `filterable` :class:`datasource`.
    """

    pass


class ParseException(Exception):
    """
    Exception that should be thrown from parsers that encounter
    exceptions they recognize while parsing. When this exception
    is thrown, the exception message and data are logged and no
    parser output data is saved.
    """

    pass


class SkipComponent(Exception):
    """
    This class should be raised by components that want to be taken out of
    dependency resolution.
    """

    pass


class ContentException(SkipComponent):
    """
    Raised whenever a :class:`datasource` fails to get data.
    """

    pass


class TimeoutException(Exception):
    """
    Raised whenever a :class:`datasource` hits the set timeout value.
    """

    pass


class ValidationException(Exception):
    """
    Raised when passes invalid arguments to Response.
    """

    def __init__(self, msg, r=None):
        if r:
            msg = "%s: %s" % (msg, r)
        super(ValidationException, self).__init__(msg)



import os
import pytest

from insights.core import dr
from insights.core.context import HostContext
from insights.core.spec_factory import listglob


@pytest.fixture(scope="module")
def sample_directory(tmpdir_factory):
    def touch(fpath):
        fd = open(fpath, "w")
        fd.close()

    tmpdir = str(tmpdir_factory.mktemp("test_listglob"))
    os.mkdir(tmpdir + "/dir1")
    os.mkdir(tmpdir + "/dir2")
    for d in ["dir1", "dir2"]:
        for f in ["file_a", "file_b"]:
            touch(tmpdir + "/" + d + "/" + f)
    return tmpdir


def run_listglob_test(sample_directory, spec):
    ctx = HostContext()
    ctx.root = sample_directory
    broker = dr.Broker()
    broker[HostContext] = ctx
    broker = dr.run([spec], broker)
    return broker


@pytest.mark.parametrize(
    "spec,result",
    [
        # return empty list if nothing matches
        (listglob("nothing/there"), []),
        (listglob("nothing/there/*"), []),
        (listglob("dir1/*", ignore="file"), []),
        # matching starts at the execution context root
        (listglob("*"), ["dir1", "dir2"]),
        # a more complex glob
        (listglob("*/*"), ["dir1/file_a", "dir1/file_b", "dir2/file_a", "dir2/file_b"]),
        # ignore filter works on paths relative to the execution context root
        (listglob("*/*", ignore="dir1.*"), ["dir2/file_a", "dir2/file_b"]),
    ]
)
def test_match_results(sample_directory, spec, result):
    broker = run_listglob_test(sample_directory, spec)
    assert broker[spec] == result


def test_ignore_does_not_use_absolute_paths(sample_directory):
    spec = listglob("dir1/file_a", ignore=sample_directory)
    broker = run_listglob_test(sample_directory, spec)
    assert broker[spec] == ["dir1/file_a"]



from insights.parsers.transparent_hugepage import ThpEnabled, ThpUseZeroPage
from insights.tests import context_wrap

ZEROPAGE_0 = "0"
ZEROPAGE_1 = "1"
ZEROPAGE_INVALID = """
bla
ble
asdf


"""

ENABLED_INVALID = """

asdf fda asdfdsaf
"""

ENABLED_MADVISE = """
always [madvise] never
"""

ENABLED_NEVER = """
always madvise [never]
"""

# testing without newlines
ENABLED_ALWAYS = """[always] madvise never"""


def test_zeropage():
    conf = ThpUseZeroPage(context_wrap(ZEROPAGE_0))
    assert conf is not None
    assert "0" == conf.use_zero_page

    conf = ThpUseZeroPage(context_wrap(ZEROPAGE_1))
    assert conf is not None
    assert "1" == conf.use_zero_page

    conf = ThpUseZeroPage(context_wrap(ZEROPAGE_INVALID))
    assert conf is not None
    assert ZEROPAGE_INVALID.replace("\n", " ").strip() == conf.use_zero_page


def test_enabled():
    conf = ThpEnabled(context_wrap(ENABLED_INVALID))
    assert conf is not None
    assert None is conf.active_option
    assert ENABLED_INVALID.strip() == conf.line

    conf = ThpEnabled(context_wrap(ENABLED_MADVISE))
    assert conf is not None
    assert "madvise" == conf.active_option
    assert ENABLED_MADVISE.strip() == conf.line

    conf = ThpEnabled(context_wrap(ENABLED_NEVER))
    assert conf is not None
    assert "never" == conf.active_option
    assert ENABLED_NEVER.strip() == conf.line

    conf = ThpEnabled(context_wrap(ENABLED_ALWAYS))
    assert conf is not None
    assert "always" == conf.active_option
    assert ENABLED_ALWAYS.strip() == conf.line



import uuid

from insights.core.plugins import make_metadata
from insights.specs import Specs
from insights.tests import InputData, archive_provider
from insights.tests.combiners.test_cloud_provider import RPMS_AWS
from insights.tests.parsers.test_aws_instance_id import AWS_ID_DOC
from insights.tests.parsers.test_client_metadata import MACHINE_ID
from insights.tests.parsers.test_etc_machine_id import ETC_MACHINE_ID
from insights.tests.parsers.test_hostname import HOSTNAME_FULL
from insights.tests.parsers.test_ip import HOSTNAME_I
from insights.util.canonical_facts import _filter_falsy, canonical_facts


def test_identity():
    assert {"foo": "bar"} == _filter_falsy({"foo": "bar"})


def test_drops_none():
    assert {"foo": "bar"} == _filter_falsy({"foo": "bar", "baz": None})


def test_drops_empty_list():
    assert {"foo": "bar"} == _filter_falsy({"foo": "bar", "baz": []})


@archive_provider(canonical_facts)
def integration_test_hit():
    data = InputData()
    data.add(Specs.installed_rpms, RPMS_AWS)
    data.add(Specs.aws_instance_id_doc, AWS_ID_DOC)
    data.add(Specs.hostname, HOSTNAME_FULL)
    data.add(Specs.ip_addresses, HOSTNAME_I)
    data.add(Specs.etc_machine_id, ETC_MACHINE_ID)
    data.add(Specs.machine_id, MACHINE_ID)
    expected = make_metadata(
        insights_id=MACHINE_ID,
        machine_id=str(uuid.UUID(ETC_MACHINE_ID)),
        ip_addresses=['10.230.230.220', '10.230.0.1'],
        fqdn=HOSTNAME_FULL,
        provider_id='i-1234567890abcdef0',
        provider_type='aws')

    yield data, expected
